
typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;

/* In DOS, Pacific C uses the large (rather than huge) memory model, so 
 * segments can't exceed 64k. Consequently we have to split the 
 * groups into 'first 64k' and 'second 64k' (or, actually, 'first 60k' 
 * and 'second 60k', so that the code gets exercised on DOS Plus builds
 * that don't have such big groups).
 *
 * Consequently, the actual way these groups are stored is an implementation
 * detail and calling code must not try to access them. Hence the different
 * names for __MSDOS__ and other platforms.
 */
typedef struct
{
#ifdef __MSDOS__
	byte *cs00;
	byte *ds00;
	byte *cs60;	/* Code group above 60k */
	byte *ds60;	/* Data group above 60k */
#else
	byte *cs;
	byte *ds;
#endif

	word cs_base;	/* Group base addresses */
	word ds_base;
	dword cs_len;	/* Group lengths on disc */
	dword ds_len;
	dword cs_max;	/* Group maximum memory allocations */
	dword ds_max;
} cmd_desc;

void zero_cmd (cmd_desc *cmdfile);
void free_cmd (cmd_desc *cmdfile);
char *alloc_cmd(cmd_desc *cmdfile, long cs_len, long ds_len);

char *load_cmd(char *filename, cmd_desc *cmdfile);
char *save_cmd(char *filename, cmd_desc *cmdfile, int absolute);

/* This is the only way to access memory inside the CMD file; all 
 * knowledge of the internal format of the segments is located here */
typedef enum
{
	CGROUP = 0,
	DGROUP = 1
} GROUP;

byte peek_cmd (cmd_desc *cmd, GROUP group, dword offset);
void poke_cmd (cmd_desc *cmd, GROUP group, dword offset, byte value);
word peekw_cmd(cmd_desc *cmd, GROUP group, dword offset);
void pokew_cmd(cmd_desc *cmd, GROUP group, dword offset, word value);

/* Copy a block of memory from one CMD to another */
void cmdcpy(cmd_desc *dest, GROUP destgroup, long destoffset,
	    cmd_desc *src, GROUP srcgroup, long srcoffset,
	    long count);
